#include <iostream>
#include <set>
#include "headers/Yerbacon.hpp"
#include <future>
#include <sstream>
using namespace std;

#include "headers/misc.hpp"
#include "headers/arguments.hpp"
#include "headers/transpiler/Target.hpp"

int main(int argc, char* argv[]) {
    if ((argc == 2) && (((string) argv[1]) == "--version")) { cout << Yerbacon::getVersion() << endl; exit(EXIT_SUCCESS); }
    string target = ".lua";
    string_view currentArg;
    bool printResult = false;
    bool parallel = false;

    if (argc > 0) {
        vector<string_view> files;
        for (signed int i = 0; i < argc; ++i)
        {
            currentArg = static_cast<string_view>(argv[i]);
            if (currentArg == ArgumentShort("printresult")) printResult = true;
            else if (currentArg == ArgumentAssignable("target")) {
                target.assign(string());
                string value = ArgumentAssignable::getValueFor(currentArg.data());
                if (!value.empty()) (target = '.') += value;
            }
            else if (currentArg == Argument("parallel")) parallel = true;
            else if (currentArg.ends_with(".ybcon")) files.push_back(currentArg);
        }
        const auto compile = [printResult, target](string_view& name) {
            const string transpiledString = transpile(parseString(getFileContent(name.data())), target);
            stringstream consoleOutput;
            if (printResult) consoleOutput << "~~~~[Yerbacon compilation result]~~~~\n\n" << "[WIP]\n" << transpiledString << "\n\n";
            name.remove_suffix(6);
            string outputFile;
            (outputFile = name).append(target);
            outputFileContent(outputFile, transpiledString);
            return consoleOutput.str();
        };
        if (!files.empty()) {
            const set<string_view> uniqueFiles(files.begin(), files.end());
            vector<future<string>> Units;
            const launch& Policy = not parallel ? launch::deferred : launch::async;
            for (string_view fileName: uniqueFiles) {
                if (fileName != "none") {
                    future<string> newAsync = async(Policy, [&fileName, &compile]() {
                        return compile(fileName);
                    });
                    Units.push_back(move(newAsync));
                }
            }
            for (auto& currentFuture: Units) cout << currentFuture.get();
        } else cout << "No valid file provided.\n";
    }
    return EXIT_SUCCESS;
}
